/*!
 * react-colrow v2.0.5
 * (c) phphe <phphe@outlook.com> (https://github.com/phphe)
 * Released under the MIT License.
 */
import _slicedToArray from '@babel/runtime/helpers/slicedToArray';
import { isNode, strRand, onDOM, offDOM, getBoundingClientRect } from 'helper-js';
import React, { createContext, useState, useRef, useMemo, useEffect, useContext } from 'react';

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') {
    return;
  }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".cr-row{\n  width: 100%;\n  overflow: hidden;\n}\n.cr-row-inner{\n  display: flex;\n  flex-wrap: wrap;\n}";
styleInject(css_248z);

// detect if need reduce col width

function isFirefox() {
  return typeof InstallTrigger !== 'undefined';
} // Safari 3.0+ "[object HTMLElementConstructor]" 


function isSafari() {
  return /constructor/i.test(window.HTMLElement) || function (p) {
    return p.toString() === "[object SafariRemoteNotification]";
  }(!window['safari'] || typeof safari !== 'undefined' && safari.pushNotification); // eslint-disable-line no-undef
} // Chrome 1 - 79


function isChrome() {
  return !!window.chrome && (!!window.chrome.webstore || !!window.chrome.runtime);
} // detect by DOM
// const div = document.createElement('div')
// document.body.appendChild(div)
// div.style.width='calc(1000px / 7)'
// const w = hp.getBoundingClientRect(div).width
// if (w * 7 > 1000) {
//   _ifNeedReduceColWidth = true
// }
// hp.removeEl(div)


var detectIfNeedReduceColWidth = (function () {
  return Boolean(!isNode() && !isChrome() && !isSafari() && !isFirefox());
});

var RowContext = createContext({});

var ifNeedReduceColWidth = detectIfNeedReduceColWidth();
var config = {
  DEFAULT_GUTTER_X: 16,
  DEFAULT_GUTTER_Y: 16,
  COL_WIDTH_REDUCE: 0.09,
  BREAK_POINTS: {
    xs: 544,
    sm: 768,
    md: 992,
    lg: 1200
  },
  ROW_HEIGHT_CALCULATION: true
};
var defaultProps = {
  gutter: [config.DEFAULT_GUTTER_X, config.DEFAULT_GUTTER_Y],
  heightCalculation: config.ROW_HEIGHT_CALCULATION,
  breakPoints: config.BREAK_POINTS
};
function Row() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  props = Object.assign(Object.assign({}, defaultProps), props);
  var className = "cr-col-".concat(strRand(6));
  var _props = props,
      gutter = _props.gutter,
      breakPoints = _props.breakPoints,
      xsGutterX = _props.xsGutterX,
      xsGutterY = _props.xsGutterY,
      smGutterX = _props.smGutterX,
      smGutterY = _props.smGutterY,
      mdGutterX = _props.mdGutterX,
      mdGutterY = _props.mdGutterY,
      lgGutterX = _props.lgGutterX,
      lgGutterY = _props.lgGutterY,
      xlGutterX = _props.xlGutterX,
      xlGutterY = _props.xlGutterY; // state

  var resolveGutterXY = function resolveGutterXY() {
    var t = Array.isArray(gutter) ? gutter.slice() : [gutter, gutter];

    if (t[0] == null) {
      t[0] = config.DEFAULT_GUTTER_X;
    }

    if (t[1] == null) {
      t[1] = config.DEFAULT_GUTTER_Y;
    }

    return t;
  };

  var initialGutter = resolveGutterXY();

  var _useState = useState(initialGutter[0]),
      _useState2 = _slicedToArray(_useState, 2),
      gutterX = _useState2[0],
      setgutterX = _useState2[1];

  var _useState3 = useState(initialGutter[1]),
      _useState4 = _slicedToArray(_useState3, 2),
      gutterY = _useState4[0],
      setgutterY = _useState4[1];

  var _useState5 = useState(),
      _useState6 = _slicedToArray(_useState5, 2),
      innerHeight = _useState6[0],
      setinnerHeight = _useState6[1]; // ref


  var inner = useRef(null);

  var updateInnerHeight = function updateInnerHeight() {
    if (inner.current != null) {
      var el = inner.current;
      var h = getBoundingClientRect(el).height;

      if (h !== innerHeight) {
        setinnerHeight(h);
      }
    }
  }; // computed


  var styleText = useMemo(function () {
    var baseStyleText = function baseStyleText(argGutterX, argGutterY) {
      if (argGutterX == null) {
        argGutterX = gutterX;
      }

      if (argGutterY == null) {
        argGutterY = gutterY;
      }

      var styleText = ".".concat(className, "{\n");
      styleText += "margin-right: -".concat(argGutterX, "px;");

      if (innerHeight == null) {
        styleText += "margin-bottom: -".concat(argGutterY, "px;");
      } else if (innerHeight !== 0) {
        styleText += "height: ".concat(innerHeight - argGutterY, "px;");
      }

      styleText += "}";
      styleText += ".".concat(className, " > .cr-row-inner{\n        width: calc(100% + ").concat(argGutterX, "px);\n      }");
      return styleText;
    };

    var styleText = baseStyleText(gutterX, gutterY); // responsive

    var bp = breakPoints;

    if (xsGutterX != null || xsGutterY != null) {
      styleText += "@media (max-width: ".concat(bp.xs, "px) {").concat(baseStyleText(xsGutterX, xsGutterY), "}");
    }

    if (smGutterX != null || smGutterY != null) {
      styleText += "@media (min-width: ".concat(bp.xs, "px) {").concat(baseStyleText(smGutterX, smGutterY), "}");
    }

    if (mdGutterX != null || mdGutterY != null) {
      styleText += "@media (min-width: ".concat(bp.sm, "px) {").concat(baseStyleText(mdGutterX, mdGutterY), "}");
    }

    if (lgGutterX != null || lgGutterY != null) {
      styleText += "@media (min-width: ".concat(bp.md, "px) {").concat(baseStyleText(lgGutterX, lgGutterY), "}");
    }

    if (xlGutterX != null || xlGutterY != null) {
      styleText += "@media (min-width: ".concat(bp.lg, "px) {").concat(baseStyleText(xlGutterX, xlGutterY), "}");
    } // 


    return "<style type=\"text/css\">".concat(styleText, "</style>").replace(/\n/g, '');
  }, [gutterX, gutterY, breakPoints, xsGutterX, xsGutterY, smGutterX, smGutterY, mdGutterX, mdGutterY, lgGutterX, lgGutterY, xlGutterX, xlGutterY, className, innerHeight]); // watch

  var updateGutterXY = function updateGutterXY() {
    var t = resolveGutterXY();
    setgutterX(t[0]);
    setgutterY(t[1]);
  };

  useEffect(updateGutterXY, [gutter]); // mounted

  useEffect(function () {
    if (props.heightCalculation) {
      updateInnerHeight();
      onDOM(window, 'resize', updateInnerHeight);
      var observer;

      if (window.MutationObserver) {
        // Select the node that will be observed for mutations
        var targetNode = document.body.parentNode;

        if (targetNode) {
          // Options for the observer (which mutations to observe)
          var _config = {
            attributes: true,
            childList: true,
            subtree: true
          }; // Callback function to execute when mutations are observed

          var callback = function callback() {
            updateInnerHeight();
          }; // Create an observer instance linked to the callback function


          observer = new MutationObserver(callback); // Start observing the target node for configured mutations

          observer.observe(targetNode, _config);
        }
      } // destroy


      return function () {
        offDOM(window, 'resize', updateInnerHeight);

        if (observer) {
          // Later, you can stop observing
          observer.disconnect();
        }
      };
    }
  }, []); // eslint-disable-line react-hooks/exhaustive-deps
  // render

  return React.createElement(RowContext.Provider, {
    value: {
      gutterX: gutterX,
      gutterY: gutterY,
      breakPoints: breakPoints,
      xsGutterX: xsGutterX,
      xsGutterY: xsGutterY,
      smGutterX: smGutterX,
      smGutterY: smGutterY,
      mdGutterX: mdGutterX,
      mdGutterY: mdGutterY,
      lgGutterX: lgGutterX,
      lgGutterY: lgGutterY,
      xlGutterX: xlGutterX,
      xlGutterY: xlGutterY
    }
  }, React.createElement("div", {
    className: "cr-row ".concat(props.className || '', " ").concat(className),
    style: props.style
  }, React.createElement("div", {
    className: "cr-row-inner",
    ref: inner
  }, props.children), React.createElement("div", {
    className: "cr-dynamic-style",
    style: {
      display: 'none'
    },
    dangerouslySetInnerHTML: {
      __html: styleText
    }
  })));
}

var css_248z$1 = ".cr-col{\n  box-sizing: border-box;\n}";
styleInject(css_248z$1);

var defaultProps$1 = {
  colWidthReduce: config.COL_WIDTH_REDUCE
};
function Col() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  props = Object.assign(Object.assign({}, defaultProps$1), props);
  var className = "cr-col-".concat(strRand(6));
  var _props = props,
      width = _props.width,
      grow = _props.grow,
      xs = _props.xs,
      xsGrow = _props.xsGrow,
      sm = _props.sm,
      smGrow = _props.smGrow,
      md = _props.md,
      mdGrow = _props.mdGrow,
      lg = _props.lg,
      lgGrow = _props.lgGrow,
      xl = _props.xl,
      xlGrow = _props.xlGrow,
      colWidthReduce = _props.colWidthReduce;
  var ctx = useContext(RowContext);
  var gutterX = ctx.gutterX,
      gutterY = ctx.gutterY,
      breakPoints = ctx.breakPoints,
      xsGutterX = ctx.xsGutterX,
      xsGutterY = ctx.xsGutterY,
      smGutterX = ctx.smGutterX,
      smGutterY = ctx.smGutterY,
      mdGutterX = ctx.mdGutterX,
      mdGutterY = ctx.mdGutterY,
      lgGutterX = ctx.lgGutterX,
      lgGutterY = ctx.lgGutterY,
      xlGutterX = ctx.xlGutterX,
      xlGutterY = ctx.xlGutterY; // computed 

  var styleText = useMemo(function () {
    var baseStyleText = function baseStyleText(width, grow, gutterX, gutterY) {
      // convert width to css text
      var widthText = function widthText(width, gutterX) {
        if (width == null) {
          width = props.width;
        }

        if (gutterX == null) {
          gutterX = ctx.gutterX;
        }

        if (typeof width === 'number') {
          if (width <= 1) {
            var reduce = ifNeedReduceColWidth ? " - ".concat(colWidthReduce, "px") : '';
            return "calc(100% * ".concat(width, " - ").concat(gutterX, "px").concat(reduce, ")");
          } else {
            return "".concat(width, "px");
          }
        } else {
          return width; // such as 100px, 100em, 10cm
        }
      };

      var empty = true;
      var styles = [];

      if (gutterX != null) {
        styles.push("margin-right: ".concat(gutterX, "px;"));
        empty = false;
      }

      if (gutterY != null) {
        styles.push("margin-bottom: ".concat(gutterY, "px;"));
        empty = false;
      }

      if (width == null && grow) {
        width = 2;
      }

      if (width != null || gutterX != null) {
        styles.push("width: ".concat(widthText(width, gutterX), ";"));
        empty = false;
      }

      if (grow != null && grow !== false) {
        if (grow === true) {
          grow = 1;
        }

        styles.push("flex-grow: ".concat(grow, ";"));
        empty = false;
      }

      var style = ".".concat(className, "{").concat(styles.join(''), "}");
      return {
        empty: empty,
        style: style
      };
    };

    var styleText = "";
    var w = width;

    if (w == null && !grow) {
      w = 1;
    }

    styleText += baseStyleText(w, grow, gutterX, gutterY).style; // responsive

    var bp = breakPoints;
    var t;
    t = baseStyleText(xs, xsGrow, xsGutterX, xsGutterY);

    if (!t.empty) {
      styleText += "\n        @media (max-width: ".concat(bp.xs, "px) {\n          ").concat(t.style, "\n        }\n      ");
    }

    t = baseStyleText(sm, smGrow, smGutterX, smGutterY);

    if (!t.empty) {
      styleText += "\n        @media (min-width: ".concat(bp.xs, "px) {\n          ").concat(t.style, "\n        }\n      ");
    }

    t = baseStyleText(md, mdGrow, mdGutterX, mdGutterY);

    if (!t.empty) {
      styleText += "\n        @media (min-width: ".concat(bp.sm, "px) {\n          ").concat(t.style, "\n        }\n      ");
    }

    t = baseStyleText(lg, lgGrow, lgGutterX, lgGutterY);

    if (!t.empty) {
      styleText += "\n        @media (min-width: ".concat(bp.md, "px) {\n          ").concat(t.style, "\n        }\n      ");
    }

    t = baseStyleText(xl, xlGrow, xlGutterX, xlGutterY);

    if (!t.empty) {
      styleText += "\n        @media (min-width: ".concat(bp.lg, "px) {\n          ").concat(t.style, "\n        }\n      ");
    } // 


    return "<style type=\"text/css\">".concat(styleText, "</style>").replace(/\n/g, '');
  }, [width, grow, xs, xsGrow, sm, smGrow, md, mdGrow, lg, lgGrow, xl, xlGrow, colWidthReduce, gutterX, gutterY, breakPoints, xsGutterX, xsGutterY, smGutterX, smGutterY, mdGutterX, mdGutterY, lgGutterX, lgGutterY, xlGutterX, xlGutterY, className, ctx.gutterX, props.width]); // render

  return React.createElement("div", {
    className: "cr-col ".concat(className, " ").concat(props.className || ''),
    style: props.style
  }, props.children, React.createElement("div", {
    className: "cr-dynamic-style",
    style: {
      display: 'none'
    },
    dangerouslySetInnerHTML: {
      __html: styleText
    }
  }));
}

var css_248z$2 = ".cr-break-row{\n  width: 100%;\n}";
styleInject(css_248z$2);

function BreakRow() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var className = "cr-break-row-".concat(strRand(6));
  var xs = props.xs,
      sm = props.sm,
      md = props.md,
      lg = props.lg,
      xl = props.xl;

  var _useContext = useContext(RowContext),
      breakPoints = _useContext.breakPoints; // computed


  var styleText = useMemo(function () {
    if (xs || sm || md || lg || xl) {
      var _styleText = ".".concat(className, "{display: none;}");

      var bp = breakPoints;

      if (xs) {
        _styleText += "\n          @media (max-width: ".concat(bp.xs, "px){\n            .").concat(className, "{display: block;}\n          }\n        ");
      }

      if (sm) {
        _styleText += "\n          @media (max-width: ".concat(bp.sm, "px) and (min-width: ").concat(bp.xs, "px){\n            .").concat(className, "{display: block;}\n          }\n        ");
      }

      if (md) {
        _styleText += "\n          @media (max-width: ".concat(bp.md, "px) and (min-width: ").concat(bp.sm, "px){\n            .").concat(className, "{display: block;}\n          }\n        ");
      }

      if (lg) {
        _styleText += "\n          @media (max-width: ".concat(bp.lg, "px) and (min-width: ").concat(bp.md, "px){\n            .").concat(className, "{display: block;}\n          }\n        ");
      }

      if (xl) {
        _styleText += "\n          @media (min-width: ".concat(bp.lg, "px){\n            .").concat(className, "{display: block;}\n          }\n        ");
      }

      return "<style type=\"text/css\">".concat(_styleText, "</style>").replace(/\n/g, '');
    }

    return '';
  }, [xs, sm, md, lg, xl, breakPoints, className]); // render

  return React.createElement("div", {
    className: "cr-break-row ".concat(className, " ").concat(props.className || ''),
    style: props.style
  }, React.createElement("div", {
    className: "cr-dynamic-style",
    style: {
      display: 'none'
    },
    dangerouslySetInnerHTML: {
      __html: styleText
    }
  }));
}

export { BreakRow, Col, Row, config };
